import { dirname, isAbsolute, relative } from 'path';
import chalk from 'chalk';
import { generateDtsContentWithSourceMap, getDtsFilePath } from './dts.js';
import { writeFileIfChanged } from './file-system.js';
import { generateSourceMappingURLComment, getSourceMapFilePath } from './source-map.js';
export function getRelativePath(fromFilePath, toFilePath) {
    const resolved = relative(dirname(fromFilePath), toFilePath);
    if (resolved.startsWith('..')) {
        return resolved;
    }
    else {
        return './' + resolved;
    }
}
export function isSubDirectoryFile(fromDirectory, toFilePath) {
    return isAbsolute(toFilePath) && toFilePath.startsWith(fromDirectory);
}
function outputWriteLog(cwd, filePath) {
    console.log('Wrote ' + chalk.green(relative(cwd, filePath)));
}
export async function emitGeneratedFiles({ filePath, tokens, distOptions, emitDeclarationMap, dtsFormatOptions, silent, cwd, isExternalFile, }) {
    const dtsFilePath = getDtsFilePath(filePath, distOptions);
    const sourceMapFilePath = getSourceMapFilePath(filePath, distOptions);
    const { dtsContent, sourceMap } = generateDtsContentWithSourceMap(filePath, dtsFilePath, sourceMapFilePath, tokens, dtsFormatOptions, isExternalFile);
    if (emitDeclarationMap) {
        const sourceMappingURLComment = generateSourceMappingURLComment(dtsFilePath, sourceMapFilePath);
        await writeFileIfChanged(dtsFilePath, dtsContent + sourceMappingURLComment);
        if (!silent)
            outputWriteLog(cwd, dtsFilePath);
        // NOTE: tsserver does not support inline declaration maps. Therefore, sourcemap files must be output.
        // blocked by: https://github.com/microsoft/TypeScript/issues/38966
        await writeFileIfChanged(sourceMapFilePath, sourceMap.toString());
        if (!silent)
            outputWriteLog(cwd, sourceMapFilePath);
    }
    else {
        await writeFileIfChanged(dtsFilePath, dtsContent);
        if (!silent)
            outputWriteLog(cwd, dtsFilePath);
    }
}
//# sourceMappingURL=index.js.map