import { constants } from 'fs';
import { access } from 'fs/promises';
import { join } from 'path';
import minimatch from 'minimatch';
export function isSystemError(value) {
    return (isObject(value) &&
        hasProp(value, 'constructor') &&
        isObject(value.constructor) &&
        hasProp(value.constructor, 'name') &&
        value.constructor.name === 'Error' &&
        hasProp(value, 'code') &&
        typeof value.code === 'string');
}
export function isObject(value) {
    return (typeof value === 'object' || typeof value === 'function') && value !== null;
}
export function hasProp(obj, prop) {
    return prop in obj;
}
export function unique(array) {
    return [...new Set(array)];
}
export function uniqueBy(arr, fn) {
    const result = [];
    const keys = new Set();
    for (const el of arr) {
        const key = fn(el);
        if (!keys.has(key)) {
            keys.add(key);
            result.push(el);
        }
    }
    return result;
}
export function sleepSync(ms) {
    const start = Date.now();
    // eslint-disable-next-line no-empty
    while (Date.now() - start < ms) { }
}
export async function exists(path) {
    try {
        await access(path, constants.F_OK);
        return true;
    }
    catch (e) {
        return false;
    }
}
export function isMatchByGlob(filePath, pattern, options) {
    return minimatch(filePath, join(options.cwd, pattern));
}
//# sourceMappingURL=util.js.map